Ext.require([
	'Ext.form.field.ComboBox',
	'Ext.data.*'
]);




Ext.define(
	'DSS.addon.HueControllerBridge',
	{
		extend: 'DSS.gridPanel',

		constructor: function(config)
		{
			var me = this;
			this.addEvents('configureBridge');
			config.customButtons =
				[
					{
						text: _('Configure HUE bridge'),
						iconCls: 'icon-manual-discovery',
						handler: function()
						{
							me.fireEvent('configureBridge', me);
						}
					}
				];
			this.callParent(arguments);
		}
	}
);


Ext.define(
	'DSS.addon.HueController.ColorPicker',
	{
		extend: 'Ext.Component',
		alias: 'hueColorPicker',

		mixins:
		{
			observable: 'Ext.util.Observable'
		},

		initialColorValue: '000000',

		clickable: true,

		border: 0,
		style:
		{
			borderStyle: 'inset',
			width: '2.5em',
			height: '2.5em'
		},

		autoEl:
		{
			tag: 'div',
			cls: 'hue-colorpicker-class'
		},

		jPickerClientPath: './jpicker/images/',

		getJPickerLocalization: function()
		{
			return {
				text:
				{
					title: _('Drag Markers To Pick A Color'),
					newColor: _('new'),
					currentColor: _('current'),
					ok: _('OK'),
					cancel: _('Cancel')
				},
				tooltips:
				{
					colors:
					{
						newColor: _('New Color - Press OK To Commit'),
						currentColor: _('Click To Revert To Original Color')
					},
					buttons:
					{
						ok: _('Commit To This Color Selection'),
						cancel: _('Cancel And Revert To Original Color')
					},
					hue:
					{
						radio: _('Set To Hue Color Mode'),
						textbox: _('Enter A Hue Value (0-360°)')
					},
					saturation:
					{
						radio: _('Set To Saturation Color Mode'),
						textbox: _('Enter A Saturation Value (0-100%)')
					},
					value:
					{
						radio: _('Set To Value Color Mode'),
						textbox: _('Enter A Value Value (0-100%)')
					},
					red:
					{
						radio: _('Set To Red Color Mode'),
						textbox: _('Enter A Red Value (0-255)')
					},
					green:
					{
						radio: _('Set To Green Color Mode'),
						textbox: _('Enter A Green Value (0-255)')
					},
					blue:
					{
						radio: _('Set To Blue Color Mode'),
						textbox: _('Enter A Blue Value (0-255)')
					},
					alpha:
					{
						radio: _('Set To Alpha Color Mode'),
						textbox: _('Enter A Alpha Value (0-100)')
					},
					hex:
					{
						textbox: _('Enter A Hex Color Value (#000000-#ffffff)'),
						alpha: _('Enter A Alpha Value (#00-#ff)')
					}
				}
			};
		},

		colorValue: null,
		ctmode: false,
		timeoutID: null,

		applyColorCSSBkImage: function(colval, el)
		{
			var rgb = hex2rgb(colval);
			var rgbstr = String(rgb[0]) + ',' + String(rgb[1]) + ',' + String(rgb[2]);
			var cssstr = Ext.String.format('radial-gradient(1.25em 1.25em, circle closest-side, rgba({0},255) 0%, rgba({0},255) 80%, rgba(100,100,100,255) 90%, rgba(100,100,100,0) 100%)', rgbstr);
			cssstr = Ext.String.format('background-image: -moz-{0}; background-image: -webkit-{0}; background-image: {0};', cssstr);
			el.dom.style.cssText += cssstr;
		},

		fillInnerHTML: function(ctmode)
		{
			try{
			var el = this.getEl();
			if (el !== undefined)
			{
				bkcolor = this.isDisabled() ? '#888888' : (ctmode ? '#FFFFFF' : this.getValue());
				this.applyColorCSSBkImage(bkcolor, el);
			}
			}catch(e){console.error(e);}
		},

		setCTMode: function(ctmode)
		{
			this.ctmode = ctmode;
			this.fillInnerHTML(ctmode);
		},

		setValue: function(color)
		{
			if (color[0] === '#')
				this.colorValue = color.substr(1);
			else
				this.colorValue = color;

			var el = this.getEl();
			if (el !== undefined)
			{
				this.applyColorCSSBkImage(this.ctmode ? '#FFFFFF' : this.getValue(), el);
			}
		},

		getValue: function()
		{
			return '#' + this.colorValue;
		},

		onRender: function()
		{
			this.callParent(arguments);

			this.fillInnerHTML(this.ctmode);

			if (this.clickable)
			{
				var me = this;
				this.getEl().on('click', function(e, t, eOpts) { me.openColorPickerWindow(); });
			}
		},

		openColorPickerWindow: function()
		{
			var m = new Ext.Window({
				title: _('Select color'),
				width: 560,
				height: 350,
				modal: true
			});

			var me = this;

			var color2color = function(color)
			{
				var all = color.val('all');
				return all && '#' + all.hex || 'transparent';
			};

			var ccancel = function(color)
			{
				if (me.timeoutID !== null)
				{
					window.clearTimeout(me.timeoutID);
					me.timeoutID = null;
				}

				me.fireEvent('change', color2color(color));

				m.destroy();
			};

			var ccommit = function(color)
			{
				if (me.timeoutID !== null)
				{
					window.clearTimeout(me.timeoutID);
					me.timeoutID = null;
				}

				var oldColorValue = me.colorValue;
				me.setValue(color2color(color));
				m.destroy();
				me.fireEvent('setValue', '#' + me.colorValue, '#' + oldColorValue);
			};

			var clive = function(color)
			{
				if (me.timeoutID !== null)
				{
					window.clearTimeout(me.timeoutID);
					me.timeoutID = null;
				}

				// Do not send change event immediately; instead, wait for 500 milliseconds
				// if the clive callback is not invoked in these 500 milliseconds, the
				// change event is triggered
				// This makes sure the change event is not triggered all the time
				// (it would otherwise be fired hundreds of times per second..)
				me.timeoutID = window.setTimeout(
					function()
					{
						me.fireEvent('change', color2color(color));
					},
					500
				);

			};

			var color = this.getValue();

			m.show();

			$(m.body.dom).jPicker(
				{
					color:
					{
						active: new $.jPicker.Color({ hex: color })
					},
					images:
					{
						clientPath: this.jPickerClientPath
					},
					localization: this.getJPickerLocalization()
				},
				ccommit, clive, ccancel
			);
		},

		constructor: function(config)
		{
			if (config.initialColorValue !== undefined)
				this.initialColorValue = config.initialColorValue;
			if (config.clickable !== undefined)
				this.clickable = !!(config.clickable);

			this.colorValue = this.initialColorValue;

			this.callParent(arguments);
			this.mixins.observable.constructor.call(this, config);

			this.addEvents('setValue');
			this.addEvents('change');
		}
	}
);


Ext.define(
	'DSS.addon.HueController.ColorTemperatureDialog',
	{
		extend: 'Ext.window.Window',
		xtype: 'colortempdialog',

		mixins:
		{
			observable: 'Ext.util.Observable'
		},

		title: _('Select color temperature'),

		width: 600,
		height: 200,

		layout: {
			type: 'vbox',
			align : 'stretch'
		},

		modal: true,

		buttons: [
			{
				text: _('OK'),
				handler: function()
				{
					var pdlg = this.findParentByType('colortempdialog');
					pdlg.fireEvent('setValue', pdlg.colortempNumber.getValue(), percent2Brightness(pdlg.brightnessNumber.getValue()));
					pdlg.destroy();
				}
			},
			{
				text: _('Cancel'),
				handler: function()
				{
					var pdlg = this.findParentByType('colortempdialog');
					pdlg.destroy();
				}
			}
		],

		items:
		[
			{
				xtype: 'container',
				layout:
				{
					type: 'hbox',
					align: 'middle'
				},
				flex: 4,
				padding: 5,
				items:
				[
					{
						xtype: 'slider',
						name: 'colortempSlider',
						align: 'middle',
						margin: '0 5 0 5',
						flex: 3,
						increment: 100,
						width: 200,
						minValue: 2000,
						maxValue: 6500,
						value: 2000,
						vertical: false
					},
					{
						xtype: 'numberfield',
						name: 'colortempNumber',
						align: 'middle',
						margin: '0 5 0 5',
						flex: 0,
						minValue: 2000,
						maxValue: 6500,
						value: 2000,
						enableKeyEvents: true,
						maxWidth: 70
					},
					{
						xtype: 'displayfield',
						value: 'K',
						margin: '0 5 0 0'
					}
				]
			},
			{
				xtype: 'container',
				layout:
				{
					type: 'hbox',
					align: 'middle'
				},
				flex: 4,
				padding: 5,
				items:
				[
					{
						xtype: 'slider',
						name: 'brightnessSlider',
						align: 'middle',
						margin: '0 5 0 5',
						flex: 3,
						increment: 1,
						width: 200,
						minValue: 0,
						maxValue: 100,
						value: 100,
						vertical: false
					},
					{
						xtype: 'numberfield',
						name: 'brightnessNumber',
						align: 'middle',
						margin: '0 5 0 5',
						flex: 0,
						minValue: 0,
						maxValue: 100,
						value: 100,
						enableKeyEvents: true,
						maxWidth: 70
					},
					{
						xtype: 'displayfield',
						value: '%',
						margin: '0 5 0 0'
					}
				]
			}
		],

		colortempSlider: null,
		colortempNumber: null,
		brightnessSlider: null,
		brightnessNumber: null,

		constructor: function(config)
		{
			this.callParent(arguments);

			var me = this;

			this.colortempSlider = this.query("[name=colortempSlider]")[0];
			this.colortempNumber = this.query("[name=colortempNumber]")[0];
			this.brightnessSlider = this.query("[name=brightnessSlider]")[0];
			this.brightnessNumber = this.query("[name=brightnessNumber]")[0];

			// field labels need to be set here, because otherwise, they
			// will be set before the translation kicks in
			this.colortempSlider.fieldLabel = _('Color temperature');
			this.brightnessSlider.fieldLabel = _('Brightness');

			if (config.colortemp !== undefined)
			{
				me.colortempSlider.setValue(config.colortemp);
				me.colortempNumber.setValue(config.colortemp);
			}

			if (config.brightness !== undefined)
			{
				me.brightnessSlider.setValue(brightness2Percent(config.brightness));
				me.brightnessNumber.setValue(brightness2Percent(config.brightness));
			}

			this.colortempSlider.on('changecomplete', function(slider, newValue, thumb, eOpts) {
				me.colortempNumber.setValue(newValue);
				me.fireEvent('change', newValue, percent2Brightness(me.brightnessNumber.getValue()));
			});
			this.colortempNumber.on('specialkey', function(field, e) {
				if (e.getKey() == e.ENTER)
				{
					me.colortempSlider.setValue(me.colortempNumber.getValue());
					me.fireEvent('change', me.colortempNumber.getValue(), percent2Brightness(me.brightnessNumber.getValue()));
				}
			});
			this.colortempNumber.on('spin', function(field, dir, opts) {
				// spin events always happen *before* the value is changed, so
				// the value needs to be adjusted to reflect the new state
				// (instead of the old one)
				var newValue = me.colortempNumber.getValue() + me.colortempNumber.step * ((dir === 'up') ? 1 : -1);
				me.colortempSlider.setValue(newValue);
				me.fireEvent('change', newValue, percent2Brightness(me.brightnessNumber.getValue()));
			});

			this.brightnessSlider.on('changecomplete', function(slider, newValue, thumb, eOpts) {
				me.brightnessNumber.setValue(newValue);
				me.fireEvent('change', me.colortempNumber.getValue(), percent2Brightness(newValue));
			});
			this.brightnessNumber.on('keypress', function(field, e) {
				if (e.getKey() == e.ENTER)
				{
					me.brightnessSlider.setValue(me.brightnessNumber.getValue());
					me.fireEvent('change', me.colortempNumber.getValue(), percent2Brightness(me.brightnessNumber.getValue()));
				}
			});
			this.brightnessNumber.on('spin', function(field, dir, opts) {
				// spin events always happen *before* the value is changed, so
				// the value needs to be adjusted to reflect the new state
				// (instead of the old one)
				var newValue = me.brightnessNumber.getValue() + me.colortempNumber.step * ((dir === 'up') ? 1 : -1);
				me.brightnessSlider.setValue(newValue);
				me.fireEvent('change', me.colortempNumber.getValue(), percent2Brightness(newValue));
			});

			this.addEvents('setValue');
			this.addEvents('change');
		}
	}
);


Ext.define(
	'DSS.addon.HueController.Light',
	{
		extend: 'Ext.panel.Panel',

		border: true,

		lightID: null,

		lightName: null,
		lightColor: null,
		lightValueDisplay: null,
		readCurrentValue: null,
		pickColor: null,
		pickColorTemperature: null,
		lightIgnored: null,
		lightBlinking: null,

		extraInfo: null,

		colortemp: 2000,
		brightness: 255,

		constructor: function(config)
		{
			var me = this;

			this.extraInfo = config.extraInfo;

			this.lightName = Ext.create(
				'Ext.form.Label',
				{
					text: '',
					disabledCls: 'x-form-label-disabled',
					align: 'middle',
					margin: '0 5 0 5',
					flex: 1
				}
			);

			this.lightColor = Ext.create(
				'DSS.addon.HueController.ColorPicker',
				{
					clickable: false,

					listeners:
					{
						change: function(newValue)
						{
							// XXX: should there be a connection between the picker and the
							// color temperature dialog, make sure this point is not reached
							// after the color temperature was set
							me.lightColor.setCTMode(false);

							var rgb = hex2rgb(newValue);
							var hsv = rgb2hsv(rgb);
							sendToBackend('testLight', { lightID: me.lightID, on: (hsv[2] !== 0), color: hsv }, null);

							me.updateLightValueDisplay(false, Math.round(hsv[2] * 255));
						}
					}
				}
			);

			this.lightValueDisplay = Ext.create(
				'Ext.form.Label',
				{
					text: '',
					disabledCls: 'x-form-label-disabled',
					align: 'middle',
					margin: '0 5 0 5',
					flex: 1
				}
			);

			this.readCurrentValue = Ext.create(
				'Ext.Button',
				{
					text: _('Current value'),
					iconCls: 'icon-read-current-value',
					margin: '0 5 0 5',
					flex: 0,
					handler: function()
					{
						sendToBackend("getHueBridgeLightAttributes", { lightID: me.lightID }, function(response)
						{
							var attributesJSON = response.attributes;
							var attributes = null;
							try
							{
								attributes = Ext.JSON.decode(attributesJSON);
								console.log("readCurrentValue: current attributes: " + attributesJSON);
							}
							catch (err)
							{
								console.log("readCurrentValue: error while parsing attributes: " + String(err));
							}

							if (attributes)
								me.setValue(attributes);
						});
					}
				}
			);

			this.pickColor = Ext.create(
				'Ext.Button',
				{
					text: _('Color'),
					iconCls: 'icon-edit-color',
					margin: '0 5 0 5',
					flex: 0,
					handler: function()
					{
						me.lightColor.openColorPickerWindow();
					}
				}
			);

			this.pickColorTemperature = Ext.create(
				'Ext.Button',
				{
					text: _('White'),
					iconCls: 'icon-edit-white',
					margin: '0 5 0 5',
					flex: 0,
					handler: function()
					{
						var setColorTempValueFunc = function(colorTemp, brightness)
						{
							// XXX: should there be a connection between the picker and the
							// color temperature dialog, make sure this point is not reached
							// after the color picker was closed
							me.lightColor.setCTMode(true);
							me.colortemp = colorTemp;
							me.brightness = brightness;

							sendToBackend('testLight', { lightID: me.lightID, on: (brightness !== 0), ct: kelvin2Mired(colorTemp), ctbri: brightness }, null);

							me.updateLightValueDisplay(true, brightness);
						};

						var dialog = Ext.create(
							'DSS.addon.HueController.ColorTemperatureDialog',
							{
								colortemp: me.colortemp,
								brightness: me.brightness,

								listeners:
								{
									'setValue': setColorTempValueFunc,
									'change': setColorTempValueFunc
								}
							}
						);
						dialog.show();
					}
				}
			);

			this.lightIgnored = Ext.create(
				'Ext.form.field.Checkbox',
				{
					boxLabel : _("Ignored"),
					flex: 1,
					align: 'middle',
					listeners:
					{
						change: function(checkBox, newValue, oldValue, opts)
						{
							var enable;
							if (me.lightBlinking === null)
								enable = !newValue;
							else
								enable = !newValue || me.lightBlinking.getValue();

							me.enableAllControls(enable);
						}
					}
				}
			);

			if (config.withBlinking)
			{
				this.lightBlinking = Ext.create(
					'Ext.form.field.Checkbox',
					{
						boxLabel : _("Blinking"),
						flex: 1,
						align: 'middle',
						listeners:
						{
							change: function(checkBox, newValue, oldValue, opts)
							{
								var enable = newValue || !(me.lightIgnored.getValue());
								me.enableAllControls(enable);
							}
						}
					}
				);
			}

			var checkboxArray = [ this.lightIgnored ];
			if (this.lightBlinking)
				checkboxArray.push(this.lightBlinking);
			var checkboxContainer = Ext.create(
				'Ext.container.Container',
				{
					items: checkboxArray,
					margin: '0 5 0 5',
					layout: 'vbox',
				}
			);

			var container = Ext.create(
				'Ext.container.Container',
				{
					items: [ this.lightName, this.lightColor, this.lightValueDisplay, this.readCurrentValue, this.pickColor, this.pickColorTemperature, checkboxContainer ],
					layout: { type: 'hbox', align: 'middle' }
				}
			);

			Ext.apply(this, { layout: 'fit', items : [ container ] });

			this.callParent(arguments);
		},

		updateLightValueDisplay: function(ct, brightness)
		{
			var percent = brightness2Percent(brightness);
			var text = "";
			if (ct)
				text = String(percent) + '%, ' + String(this.colortemp) + 'K';
			else
				text = String(percent) + '%';

			this.lightValueDisplay.setText(text);
		},

		enableAllControls: function(state)
		{
			var setWidgetEnabled = function(widget, b)
			{
				if (widget)
				{
					if (b)
						widget.enable();
					else
						widget.disable();
				}
			};

			setWidgetEnabled(this.lightColor, state);
			setWidgetEnabled(this.readCurrentValue, state);
			setWidgetEnabled(this.pickColor, state);
			setWidgetEnabled(this.pickColorTemperature, state);
		},

		reset: function()
		{
			this.lightColor.setCTMode(false);
			this.lightID = null;
			this.updateLightValueDisplay(false, 255);
			this.lightName.setValue('');
			this.lightColor.setValue('#ffffff');
			this.lightIgnored.setValue(true);
			if (this.lightBlinking)
				this.lightBlinking.setValue(false);
		},

		setValue: function(value)
		{
			this.lightID = value.id;

			this.lightColor.setCTMode(value.useCT);

			if (value.ct !== undefined)
				this.colortemp = mired2Kelvin(value.ct);
			if (value.ctbri !== undefined)
				this.brightness = value.ctbri;

			var hexColor = rgb2hex(hsv2rgb(value.color));
			if (!value.on)
			{
				hexColor = '#000000';
				this.brightness = 0;
			}

			this.updateLightValueDisplay(value.useCT, value.useCT ? this.brightness : Math.round(value.color[2] * 255.0));

			if (value.name !== undefined)
				this.lightName.setText(String(value.name));
			this.lightColor.setValue(hexColor);
			this.lightIgnored.setValue(value.ignored);
			if (this.lightBlinking && (value.blinking !== undefined))
				this.lightBlinking.setValue(value.blinking);

			this.enableAllControls(value.blinking || !(value.ignored));
		}
	}
);


Ext.define(
	'DSS.addon.HueController.Lights',
	{
		extend: 'DSS.component.Base',
		mixins:
		{
			observable : 'Ext.util.Observable'
		},



		extraInfo: null,
		lightControls: [],
		lightsNeedFiltering: false,
		zoneID: null,
		flex: 1,
		padding: "5 5 10 5", // for layout reasons, the bottom padding needs 5 additional pixels

		constructor: function(config)
		{
			var me = this;
			this.extraInfo = config.extraInfo;
			this.lightsNeedFiltering = (config.lightsNeedFiltering === undefined) ? false : config.lightsNeedFiltering;

			this.lightsPanel = Ext.create(
				'Ext.panel.Panel',
				{
					title: _('Lights'),
					overflowY: 'scroll',
					flex: 1,
					items: [ ]
				}
			);

			Ext.apply(this, { layout: 'fit', items : [ this.lightsPanel ] });

			this.callParent(arguments);
		},

		clearOldLightControls: function()
		{
			this.lightsPanel.removeAll();
			this.lightControls.length = 0;
		},

		addAllLights: function()
		{
			if (!this.extraInfo.hueBridgeInfo)
			{
				console.log('addAllLights: no HUE bridge information found - no lights to add');
				return;
			}

			var me = this;
			var lights = this.extraInfo.hueBridgeInfo.lights;

			if (this.lightsNeedFiltering)
			{
				var lightsInZone = ((me.zoneID === null) || (this.extraInfo.lightsInZonesTable === null)) ? null : this.extraInfo.lightsInZonesTable[me.zoneID];
				if (lightsInZone)
				{
					lights = lights.filter(function(element, index, array)
					{
						return lightsInZone[element.lightID] !== undefined;
					});
				}
				else
					lights = [];
			}

			Ext.Array.each(lights, function(light)
			{
				var ldata = { id: light.lightID, name: light.name, ignored: true, color: [1, 0, 1], on: false };
				me.addLight(ldata);
			});
		},

		addLight: function(value)
		{
			var lightControl = Ext.create('DSS.addon.HueController.Light', { extraInfo: this.extraInfo, withBlinking: (this.zoneID === null) });

			if (value)
				lightControl.setValue(value);
			else
				lightControl.reset();

			this.lightControls.push(lightControl);
			this.lightsPanel.add(lightControl);
		},

		reset: function()
		{
			this.clearOldLightControls();
			// NOT adding all lights if filtering is required, since then, the lights to add depend on the room
			if (!this.lightsNeedFiltering)
				this.addAllLights();
		},

		isValid: function()
		{
			return true;
		},

		setZoneID: function(zoneID)
		{
			var newZone = (this.zoneID !== zoneID);
			this.zoneID = zoneID;
			if (this.lightsNeedFiltering && newZone)
			{
				this.clearOldLightControls();
				if (zoneID !== null)
					this.addAllLights();
			}
		},

		setValue: function(value)
		{
			var me = this;
			var lights = value;

			this.clearOldLightControls();
			this.addAllLights();

			Ext.Array.each(lights, function(light)
			{
				Ext.Array.each(me.lightControls, function(lightControl)
				{
					if (light.id === lightControl.lightID)
					{
						lightControl.setValue({
							id: light.id,
							name: lightControl.lightName.text,
							color: light.color,
							on: light.on,
							blinking: light.blinking,
							useCT: light.useCT,
							ct: light.ct,
							ctbri: light.ctbri,
							ignored: (light.ignored !== undefined) ? light.ignored : false
						});
						return false;
					}
				});
			});
		},

		getValue: function()
		{
			var lights = [];

			Ext.Array.each(this.lightControls, function(lightControl)
			{
				var rgb = hex2rgb(lightControl.lightColor.getValue());
				var hsv = rgb2hsv(rgb);
				console.log("rgb: " + Ext.JSON.encode(rgb) + " hsv: " + Ext.JSON.encode(hsv));

				// full black equals explicit "off" state
				var on = lightControl.lightColor.ctmode ? (lightControl.brightness !== 0) : (lightControl.lightColor.getValue() !== "#000000");

				var light =
				{
					id: lightControl.lightID,
					on: on,
					color: hsv,
					blinking: lightControl.lightBlinking ? lightControl.lightBlinking.getValue() : false,
					useCT: lightControl.lightColor.ctmode,
					ct: kelvin2Mired(lightControl.colortemp),
					ctbri: lightControl.brightness,
					ignored: lightControl.lightIgnored.getValue()
				};
				lights.push(light);
			});

			return lights;
		}
	}
);


function getFilteredZoneArray()
{
	// Zone ID 0 refers to "All Rooms" - filter this one out before passing on the array
	var zoneArray = dss.staticDataModel.getZoneArray();
	zoneArray = zoneArray.filter(function(element, index, array)
	{
		return element.ZoneID !== 0;
	});
	return zoneArray;
}


Ext.define(
	'DSS.addon.HueController.ZoneCombobox',
	{
		extend: 'Ext.form.field.ComboBox',
		alias : 'widget.zoneComboBox',
		queryMode: 'local',
		displayField: 'name',
		valueField: 'ZoneID',
		store: Ext.create(
			'Ext.data.Store',
			{
				fields: ['ZoneID', 'name'],
				data: getFilteredZoneArray()
			}
		)
	}
);


Ext.define(
	'DSS.addon.HueController.LightView',
	{
		extend: 'Ext.grid.column.Column',
		alias : 'widget.lightview',

		renderer: function(val, metadata, record, rowIdx, colIdx, store, view)
		{
			var expectedLightID = metadata.column.lightID;

			var lightSettings = val;
			for (var lidx = 0; lidx < lightSettings.length; ++lidx)
			{
				var light = lightSettings[lidx];
				if (light.id === expectedLightID)
				{
					if ((light.ignored !== undefined) && light.ignored)
						break;
					if (light.predefined)
						return '<div class="x-form-label-disabled">' + _('Off') + '</div>';
					if (!light.on)
						return _('Off');
					else
					{
						var html = '<table style="padding: 0pt; margin: 0pt;"><tbody><tr>';

						var text = '';
						var rgb;

						if (light.useCT)
						{
							var kelvin = mired2Kelvin(light.ct);
							rgb = [255, 255, 255];
							text = String(brightness2Percent(light.ctbri)) + '%, ' + String(kelvin) + ' K';
						}
						else
						{
							rgb = hsv2rgb(light.color);
							text = String(brightness2Percent(Math.round(light.color[2] * 255))) + '%';
						}

						if (light.blinking)
							text += ' ' + _('Blinking');

						var rgbstr = String(rgb[0]) + ',' + String(rgb[1]) + ',' + String(rgb[2]);
						var cssstr = Ext.String.format('radial-gradient(1.25em 1.25em, circle closest-side, rgba({0},255) 0%, rgba({0},255) 80%, rgba(100,100,100,255) 90%, rgba(100,100,100,0) 100%)', rgbstr);
						cssstr = Ext.String.format('background-image: -moz-{0}; background-image: -webkit-{0}; background-image: {0};', cssstr);

						html += '<td style="border: 0px; width: 2.5em; height: 2.5em; margin: 0pt; ' + cssstr + '" >';
						html += '<span style="margin-left: 3.5em;">' + text + '</span>';

						html += '</td>';

						html += '</tr></tbody></table>';

						return html;
					}
				}
			}

			return '--';
		}
	}
);


Ext.define(
	'DSS.addon.HueController',
	{
		extend : 'DSS.addon.Framework',
		fExpertMode: false,
		config :
		{
			appName : _('Hue Controller'),
			appId : 'hue-control-2',
			appVersion : '0.1',
			//appIcon : 'default_icon.png'
		},

		getHelp: function()
		{
            return Ext.create('Ext.Component', {
                loader : {
                    url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                    autoLoad : true,
                    disableCaching : false
                }
            });
		},

		presetsPanel: null,
		globalActivitiesPanel: null,
		hueBridgePanel: null,
		mainAddonPanel: null,

		presetsGrid: null,
		globalActivitiesGrid: null,
		hueBridgeGrid: null,

		extraInfo: { hueBridgeInfo: null, lightsInZonesTable: null, presetActivities: null, globalActivities: null },

		removeDisablePresetButton: function(grid)
		{
			if (grid.btnDisable != null)
			{
				grid.btnDisable.hide();
				grid.btnEnable.hide();
			}
		},

		findLightName: function(lightID)
		{
			if (!this.extraInfo.hueBridgeInfo)
				return null;

			var lightName = null;
			Ext.Array.each(this.extraInfo.hueBridgeInfo.lights, function(entry)
			{
				if (entry.lightID == lightID)
				{
					lightName = entry.name;
					return false;
				}
			});

			return lightName;
		},

		updateLightColumns: function(grid)
		{
			var me = this;

			if (this.extraInfo.hueBridgeInfo)
			{
				var columnsToRemove = [];

				var existingColumns = grid.headerCt.getGridColumns();
				Ext.Array.each(existingColumns, function(column)
				{
					if (column.initialConfig.dataIndex === "lights")
					{
						columnsToRemove.push(column);
						console.log("remove existing light column " + column.initialConfig.header);
					}
				});

				Ext.Array.each(columnsToRemove, function(column)
				{
					grid.headerCt.remove(column);
				});

				Ext.Array.each(this.extraInfo.hueBridgeInfo.lights, function(entry)
				{
					console.log("New column: ID " + String(entry.lightID) + " name " + entry.name);
					var column = Ext.create(
						'DSS.addon.HueController.LightView',
						{
							lightID: entry.lightID,
							header: entry.name,
							flex: 2,
							minWidth: 100,
							dataIndex: 'lights'
						}
					);
					grid.headerCt.add(column);
				});
				grid.getView().refresh();
			}
		},

		loadPresetsParameter: function(grid)
		{
			var me = this;
			grid.setStatus(_('Loading ...'));
			this.removeDisablePresetButton(grid);
			sendToBackend('getAllActivities', { activityType: 'preset' }, function(response)
			{
				var activities = Ext.JSON.decode(response.activities);
				me.extraInfo.presetActivities = activities;

				// even though there is a "triggers" array, it always contains
				// exactly one entry - the array exists to be compatible with
				// existing trigger UI elements
				var gridParams = [];
				var zonesSeen = {};
				var zones = dss.staticDataModel.getZoneArray();
				for (var activityID in activities)
				{
					var activity = activities[activityID];
					var zoneID = activity.triggers[0].zone;

					var zoneName = '';
					for (var i = 0; i < zones.length; ++i)
					{
						var zone = zones[i];
						if (zone.ZoneID === zoneID)
						{
							zoneName = zone.name;
							break;
						}
					}

					// add dummy preset params for showing the implicit "off" preset
					if (!zonesSeen[zoneID])
					{
						var dummyLights = [];
						for (var lidx = 0; lidx < activity.lights.length; ++lidx)
						{
							dummyLights.push({
								id: activity.lights[lidx].id,
								predefined: true,
								on: false
							});
						}

						console.log("dummyLights: " + Ext.JSON.encode(dummyLights));
						console.log("lights: " + Ext.JSON.encode(activity.lights));

						gridParams.push(
							{
								activityID: -1, // -1 = "off"
								disabled: false,
								predefined: true,

								// special value to inform the column renderer that this is the "off" preset
								triggers: [{ zone: zoneID, scene: -1, group: -1 }],

								zoneID: zoneID,
								zoneName: zoneName,
								lights: dummyLights
							}
						);
					}
					zonesSeen[zoneID] = true;

					// add the actual preset params
					gridParams.push(
						{
							activityID: activityID,
							disabled: false,
							triggers: activity.triggers,
							zoneID: zoneID,
							zoneName: zoneName,
							lights: activity.lights,
						}
					);
				}

				grid.loadParameter(gridParams);

				grid.setStatus(_('Ready'));
			});
		},

		loadGlobalActivityParameter: function(grid)
		{
			var me = this;

			grid.setStatus(_('Loading ...'));

			sendToBackend('getAllActivities', { activityType: 'globalActivity' }, function(response)
			{
				var activities = Ext.JSON.decode(response.activities);
				me.extraInfo.globalActivities = activities;

				var gridParams = [];

				for (var activityID in activities)
				{
					var activity = activities[activityID];
					gridParams.push(
						{
							disabled: !(activity.activated),
							activityID: activityID,
							triggers: activity.triggers,
							lights: activity.lights
						}
					);
				}

				grid.loadParameter(gridParams);

				grid.setStatus(_('Ready'));
			});
		},

		loadHueBridgeParameter: function(grid)
		{
			grid.setStatus(_('Loading ...'));

			this.removeDisablePresetButton(grid);

			if (this.extraInfo.hueBridgeInfo && this.extraInfo.hueBridgeInfo.lights)
			{
				var gridParams = [];
				for (var i = 0; i < this.extraInfo.hueBridgeInfo.lights.length; ++i)
				{
					var light = this.extraInfo.hueBridgeInfo.lights[i];
					gridParams.push(
						{
							lightID: light.lightID,
							lightName: light.name,
							zoneID: light.zoneID
						}
					);
				}

				grid.loadParameter(gridParams);
			}
			else
			{
				grid.loadParameter([]);
			}
			grid.setStatus(_('Ready'));

		},

		checkIfAnyRoomIsAssociated : function()
		{
			if (this.extraInfo.hueBridgeInfo && this.extraInfo.hueBridgeInfo.lights)
			{
				// create a set of zone IDs , using an object with dummy values
				var zoneIDs = {};
				var zones = dss.staticDataModel.getZoneArray();
				for (var i = 0; i < zones.length; ++i)
					zoneIDs[zones[i].ZoneID] = true;

				// check each light in the bridge, and see if at least one of their zone IDs is known
				for (var i = 0; i < this.extraInfo.hueBridgeInfo.lights.length; ++i)
				{
					var light = this.extraInfo.hueBridgeInfo.lights[i];
					if (zoneIDs[light.zoneID] !== undefined)
						return true;
				}
			}

			return false;
		},

		getContent : function()
		{
			var me = this;

			var presetsLightSettingsControl = Ext.create('DSS.addon.HueController.Lights',
			{
				name: 'lights',
				extraInfo: me.extraInfo,
				lightsNeedFiltering: true
			});
			var presetsTriggerControl = Ext.create('DSS.addon.HueController.SelectPreset',
			{
				extraInfo: me.extraInfo,
				onNewZoneIDSelected: function(zoneID)
				{
					presetsLightSettingsControl.setZoneID(zoneID);
				}
			});

			var presetsPanelConfig =
			{
				title: _('Presets'),
				layout: 'fit',
				disabledProperty: 'disabled',
				inactiveField: 'predefined',
				grouperField: 'zoneName',
				storeConfig:
				{
					storeId : 'presetsStore',
					fields :
					[
						{ name: 'activityID' },
						{ name: 'disabled' },
						{ name: 'predefined' },
						{ name: 'triggers', sortType: 'asUCText' },
						{ name: 'zoneID' },
						{ name: 'zoneName' },
						{ name: 'lights' }
					]
				},
				columns:
				[
					{
						header: _('Preset'),
						dataIndex: 'triggers',
						xtype: 'textcolumn',
						maintainFlex: true,
						minWidth: 200,
						flex: 3,
						customColumnRenderer: function(val, metadata, record, rowIdx, colIdx, store, view)
						{
							var trigger = val[0];
							if ((trigger.scene === -1) || (trigger.group === -1))
								return '<div class="x-form-label-disabled">' + _('Off') + '</div>';
							else
								return getCustomNameForScene(trigger.scene, trigger.group, trigger.zone);
						}
					}
				],
				buttonText:
				{
					newButton: _('Add room preset'),
					editButton:  _('Edit room preset'),
					deleteButton: _('Delete preset'),
					testButton: _('Test preset')
				},
				editorConfig:
				{
					title: _('Edit room preset'),
					flex: 1,
					tabControls:
					[
						{
							tabLabel: _('Trigger'),
							tabDescription: _('Please select the room and preset for these light settings.'),
							content:
							[
								{
									type: 'custom',
									id : 'triggers',
									flex : true,
									control: presetsTriggerControl
								}
							]
						},
						{
							tabLabel: _('Light settings'),
							tabDescription: _('Please select the light settings of your HUE light bulbs for this room and preset.'),
							content:
							[
								{
									type: 'hidden',
									id: 'activityID'
								},
								{
									type: 'hidden',
									id: 'zoneID'
								},
								{
									type: 'hidden',
									id: 'zoneName'
								},
								{
									type: 'custom',
									id: 'lights',
									flex : true,
									control: presetsLightSettingsControl
								}
							]
						}
					],
					listeners:
					{
						save: function()
						{
							console.log('User saved preset values - setting zone ID of lights control to null');
							presetsLightSettingsControl.setZoneID(null);
						},
						cancel: function()
						{
							console.log('User canceled preset editing - setting zone ID of lights control to null');
							presetsLightSettingsControl.setZoneID(null);
						}
					}
				},
				listeners:
				{
					'loadDataStore': function(grid)
					{
						console.log("preset: loadDataStore");
						me.presetsGrid = grid;
						me.loadPresetsParameter(grid);
					},
					'deleteEntry': function(grid, values)
					{
						var activityIDStr = values.get('activityID');
						console.log("preset: deleteEntry: " + activityIDStr);

						var payload =
						{
							activityID: parseInt(activityIDStr),
							activityType: 'preset'
						};

						sendToBackend('deleteActivity', payload, function(response)
						{
							me.loadPresetsParameter(grid);
						});
					},
					'saveEntry': function(grid, values)
					{
						grid.setStatus(_('save'));
						if (!me.extraInfo.hueBridgeInfo)
						{
							console.log('presets saveEntry: no HUE bridge information found - not saving anything');
							return;
						}

						console.log("preset: saveEntry: " + Ext.JSON.encode(values));

						var payload =
						{
							activityID: values.activityID,
							activityType: 'preset',
							activity:
							{
								triggers: values.triggers,
								lights: values.lights
							}
						};

						sendToBackend('setActivity', payload, function(response)
						{
							me.loadPresetsParameter(grid);
						});
					},
					'testEntry': function(grid, values)
					{
						var activityIDStr = values.get('activityID');

						var payload =
						{
							activityID: parseInt(activityIDStr),
							activityType: 'preset'
						};

						sendToBackend('testActivity', payload, null);
					}
				}
			};

			var setGlobalActivityFunc = function(grid, values)
			{
				grid.setStatus(_('save'));
				if (!me.extraInfo.hueBridgeInfo)
				{
					console.log('globalActivity saveEntry: no HUE bridge information found - not saving anything');
					return;
				}

				console.log("setGlobalActivityFunc : " + Ext.JSON.encode(values));

				var payload =
				{
					activityID: values.activityID,
					activityType: 'globalActivity',
					activity:
					{
						activated: !(values.disabled),
						triggers: values.triggers,
						lights: values.lights
					}
				};

				sendToBackend('setActivity', payload, function(response)
				{
					me.loadGlobalActivityParameter(grid);
				});
			};

			var globalActivitiesPanelConfig =
			{
				title: _('Global activities'),
				layout: 'fit',
				disabledProperty: 'disabled',
				storeConfig:
				{
					storeId : 'globalActivitiesStore',
					fields :
					[
						{ name: 'disabled', sortType: 'asBoolean' },
						{ name: 'activityID' },
						{ name: 'triggers', sortType: 'asTrigger' },
						{ name: 'lights' }
					]
				},
				columns:
				[
					{
						header : _('Active'),
						dataIndex : 'disabled',
						resizable : false,
						xtype: 'disabledcolumn',
						width : 50
					},
					{
						header: _('Activity'),
						dataIndex: 'triggers',
						xtype: 'triggercolumn',
						maintainFlex: true,
						flex: 3
					}
				],
				buttonText:
				{
					newButton: _('Add activity'),
					editButton:  _('Edit activity'),
					deleteButton: _('Delete activity'),
					disableButton : _('Deactivate activity'),
					enableButton: _('Activate activity'),
					testButton: _('Test activity')
				},
				editorConfig:
				{
					title: _('Edit activity'),
					flex: 1,
					tabControls:
					[
						{
							tabLabel: _('Trigger'),
							tabDescription: _('Please select the required activity. You can choose more than one activity as trigger.'),
							content:
							[
								{
									type: 'TriggerConfigurator',
									id : 'triggers',
									flex : true
								}
							]
						},
						{
							tabLabel: _('Light settings'),
							tabDescription: _('Please select the light settings of your HUE light bulbs for this activity.'),
							content:
							[
								{
									type: 'hidden',
									id: 'activityID'
								},
								{
									type: 'hidden',
									id: 'disabled'
								},
								{
									type: 'custom',
									id: 'lights',
									flex: true,
									control: Ext.create('DSS.addon.HueController.Lights',
									{
										name: 'lights',
										extraInfo: me.extraInfo
									})
								}
							]
						}
					],
					staticControls:
					[
					]
				},
				listeners:
				{
					'loadDataStore': function(grid)
					{
						console.log("globalActivity: loadDataStore");
						me.loadGlobalActivityParameter(grid);
						me.globalActivitiesGrid = grid;
					},
					'enableEntry': function(grid, values)
					{
						setGlobalActivityFunc(grid, { activityID: values.get('activityID'), disabled: false, triggers: values.get('triggers'), lights: values.get('lights') } );
					},
					'disableEntry': function(grid, values)
					{
						setGlobalActivityFunc(grid, { activityID: values.get('activityID'), disabled: true, triggers: values.get('triggers'), lights: values.get('lights') } );
					},
					'deleteEntry': function(grid, values)
					{
						var activityIDStr = values.get('activityID');
						console.log("globalActivity: deleteEntry: " + activityIDStr);

						var payload =
						{
							activityID: parseInt(activityIDStr),
							activityType: 'globalActivity'
						};

						sendToBackend('deleteActivity', payload, function(response)
						{
							me.loadGlobalActivityParameter(grid);
						});
					},
					'saveEntry': function(grid, values)
					{
						setGlobalActivityFunc(grid, values);
					},
					'testEntry': function(grid, values)
					{
						var activityIDStr = values.get('activityID');

						var payload =
						{
							activityID: parseInt(activityIDStr),
							activityType: 'globalActivity'
						};

						sendToBackend('testActivity', payload, null);
					}
				}
			};


			var hueBridgePanelConfig =
			{
				title: _('Devices'),
				layout: 'fit',
				storeConfig:
				{
					storeId : 'hueBridgeStore',
					fields :
					[
						{ name: 'lightID' },
						{ name: 'lightName', sortType: 'asUCText' },
						{ name: 'zoneID' }
					]
				},
				columns:
				[
					{
						header: _("Light"),
						dataIndex: 'lightName',
						flex: 1
					},
					{
						header: _("Room"),
						dataIndex: 'zoneID',
						flex: 2,
						xtype: 'textcolumn',
						customColumnRenderer: function(val, metadata, record, rowIdx, colIdx, store, view)
						{
							// TODO: the zone array should be cached - but what if it is updated?
							// How to update a stale zone array cache?
							var zones = dss.staticDataModel.getZoneArray();
							try
							{
								for (var i = 0; i < zones.length; ++i)
								{
									var zone = zones[i];
									if (zone.ZoneID === val)
										return zone.name;
								}
							}
							catch(err)
							{
								console.log("hueBridgePanelConfig: error while rendering room column: " + String(err));
							}

							return "&lt;" + _("no room assigned") + "&gt;";
						}
					}
				],
				buttonText:
				{
					editButton:  _('Edit device settings')
				},
				editorConfig:
				{
					title: _('Device settings'),
					staticControls:
					[
						{
							id: 'lightID',
							type: 'hidden'
						},
						{
							id: 'lightName',
							type: 'text',
							label: _('Light'),
							readOnly: true
						},
						{
							id: 'zoneID',
							type: 'custom',
							control: Ext.create('DSS.addon.HueController.ZoneCombobox',
							{
								extraInfo: me.extraInfo,
								fieldLabel: _('Room')
							})
						}
					]
				},
				listeners:
				{
					'configureBridge': function(grid)
					{
						Ext.Msg.prompt(
							_('HUE bridge IP address'),
							_('Please enter the IP address of the HUE bridge (or just press enter for autodetection)'),
							function(btn, text)
							{
								if (btn == 'ok')
								{
									var newIPAddress = text;
									console.log("HUE bridge: new IP address: " + String(newIPAddress));
									Ext.getBody().mask(_("Updating HUE bridge information"), 'x-mask-loading');
									sendToBackend("setHueBridgeAddress", { ipaddress: newIPAddress }, function(response)
									{
										me.updateHueBridge(response);
									});
								}
							}
						);
					},
					'findNewLights': function(grid)
					{
						sendToBackend("findNewLights", { }, null);
						Ext.MessageBox.alert(_('HUE bridge'), _('HUE bridge is now searching for new lights. Please press Refresh view below after one minute.'));
					},
					'loadDataStore': function(grid)
					{
						me.hueBridgeGrid = grid;
						me.updateHueBridge();
					},
					'saveEntry': function(grid, values)
					{
						Ext.getBody().mask(_("Getting HUE bridge information"), 'x-mask-loading');
						grid.setStatus(_('save'));
						console.log(Ext.JSON.encode(values));

						var payload =
						{
							lightID: values.lightID,
							zoneID: values.zoneID
						};

						sendToBackend("setHueLightZoneAssociation", payload, function(response)
						{
							me.updateHueBridge(response);
							dss.staticDataModel.refreshDataModel();
						});
					}
				}
			};


			me.presetsPanel = Ext.create('DSS.gridPanel', presetsPanelConfig);
			me.globalActivitiesPanel = Ext.create('DSS.gridPanel', globalActivitiesPanelConfig);
			me.hueBridgePanel = Ext.create('DSS.addon.HueControllerBridge', hueBridgePanelConfig);

			var oldNewPresetHandler = me.presetsPanel.btnNew.handler;
			me.presetsPanel.btnNew.setHandler(function() {
				if (me.checkIfAnyRoomIsAssociated())
					oldNewPresetHandler();
				else
					Ext.MessageBox.alert({ title: _('Presets'), msg: _('None of the rooms are associated with HUE lights - cannot add a preset'), buttons: Ext.MessageBox.OK, icon: Ext.MessageBox.ERROR, modal: true });
			});
			me.presetsPanel.on({
				selectionchange: function(view, selections, options)
				{
					me.removeDisablePresetButton(me.presetsPanel);
				}
			});

			me.mainAddonPanel = Ext.create('Ext.tab.Panel', {
				layout: 'fit',
				cls: 'generalTab',
				items: [ me.presetsPanel, me.globalActivitiesPanel, me.hueBridgePanel ]
			});

			me.updateHueBridge();

			return me.mainAddonPanel;
		},

		updateHueBridge: function(response)
		{
			var me = this;

			var getHueBridgeInfo = null, requestLinkButtonPress = null;

			var requestLinkButtonPress = function()
			{
				Ext.MessageBox.alert(_('HUE bridge'), _('Please press the link button on the HUE bridge, then press OK.'), function(btn, text, opt)
				{
					if (btn == 'ok')
						getHueBridgeInfo('registerHueBridge');
					else
						me.hueBridgeGrid.setStatus(_('User did not press link button on HUE bridge - please configure again'));
				});
			};

			var processHueBridgeInfoResponse = function(info)
			{
				me.extraInfo.hueBridgeInfo = null;
				me.extraInfo.lightsInZonesTable = null;

				var registerStatus = parseInt(info.registerStatus);

				switch (registerStatus)
				{
					case 0:
					{
						info = Ext.JSON.decode(info.info);
						console.log("processHueBridgeInfoResponse: bridge info: " + Ext.JSON.encode(info));

						me.extraInfo.hueBridgeInfo = info;

						var lightsInZonesTable = {};
						for (var i = 0; i < info.lights.length; ++i)
						{
							var light = info.lights[i];
							var lightsInZone = null;
							if (lightsInZonesTable[light.zoneID] === undefined)
							{
								lightsInZone = {};
								lightsInZonesTable[light.zoneID] = lightsInZone;
							}
							else
								lightsInZone = lightsInZonesTable[light.zoneID];

							lightsInZone[light.lightID] = true; // value is not actually used; the object is used as a set instead
						}
						me.extraInfo.lightsInZonesTable = lightsInZonesTable;

						me.hueBridgeGrid.setStatus(_('Ready'));

						break;
					}

					case 1:
					{
						requestLinkButtonPress();
						break;
					}

					default:
					{
						var hueBridgeErrorText1 = _('Error while connecting to the HUE bridge - please reconfigure');

						var hueBridgeErrorText2 = '';
						switch (registerStatus)
						{
							case 3:
								hueBridgeErrorText2 = _('Could not retrieve information from the HUE bridge');
								break;
							case 4:
								hueBridgeErrorText2 = _('No HUE bridge was found');
								break;
							default:
								hueBridgeErrorText2 = _('HUE error information') + ': ' + String(info.errorMsg);
						}

						var text = hueBridgeErrorText1 + '<br/>' + hueBridgeErrorText2;

						Ext.MessageBox.alert({ title: _('HUE bridge'), msg: text, buttons: Ext.MessageBox.OK, icon: Ext.MessageBox.ERROR, modal: true });
						me.hueBridgeGrid.setStatus(text);
					}
				}

				me.updateLightColumns(me.presetsGrid);
				me.updateLightColumns(me.globalActivitiesGrid);

				me.loadPresetsParameter(me.presetsGrid);
				me.loadGlobalActivityParameter(me.globalActivitiesGrid);
				me.loadHueBridgeParameter(me.hueBridgeGrid);

				Ext.getBody().unmask();
			};

			getHueBridgeInfo = function(requestCall)
			{
				Ext.getBody().mask(_("Getting HUE bridge information"), 'x-mask-loading');
				sendToBackend(requestCall, {}, function(response)
				{
					processHueBridgeInfoResponse(response);
				});
			}

			// response == null is valid here
			if (response)
				processHueBridgeInfoResponse(response);
			else
				getHueBridgeInfo('getHueBridgeInfo');
		}
	}
);


Ext.onReady(
	function()
	{
		dss.buildUpLang(['locale/{languageSuffix}/hue-controller.po']);
		var huecontrol = Ext.create('DSS.addon.HueController');
		huecontrol.initPage();
	}
);

